@@ -30,6 +30,7 @@
 #include <unictype.h>
 #include <unicase.h>
 
+#include "tracker-language.h"
 #include "tracker-parser.h"
 #include "tracker-parser-utils.h"
 
@@ -40,6 +41,9 @@ typedef enum {
 	TRACKER_PARSER_WORD_TYPE_OTHER_NO_UNAC,
 } TrackerParserWordType;
 
+/* If string lenth less than this value, allocating from the stack */
+#define MAX_STACK_STR_SIZE 8192
+
 /* Max possible length of a UTF-8 encoded string (just a safety limit) */
 #define WORD_BUFFER_LENGTH 512
 
@@ -84,7 +88,7 @@ get_word_info (TrackerParser         *parser,
 
 	/* Get first character of the word as UCS4 */
 	first_unichar_len = u8_strmbtouc (&first_unichar,
-	                                  &(parser->txt[parser->cursor]));
+	                                  (const guchar *) &(parser->txt[parser->cursor]));
 	if (first_unichar_len <= 0) {
 		/* This should only happen if NIL was passed to u8_strmbtouc,
 		 *  so better just force stop here */
@@ -106,7 +110,7 @@ get_word_info (TrackerParser         *parser,
 		i = parser->cursor + first_unichar_len;
 		while (1) {
 			/* Text bounds reached? */
-			if (i >= parser->txt_size)
+			if (i >= (gsize) parser->txt_size)
 				break;
 			/* Proper unicode word break detected? */
 			if (parser->word_break_flags[i])
@@ -159,7 +163,7 @@ get_word_info (TrackerParser         *parser,
 /* The input word in this method MUST be normalized in NFKD form,
  * and given in UTF-8, where str_length is the byte-length
  * (note: there is no trailing NUL character!) */
-gboolean
+static gboolean
 tracker_parser_unaccent_nfkd_string (gpointer  str,
                                      gsize    *str_length)
 {
@@ -181,7 +185,7 @@ tracker_parser_unaccent_nfkd_string (gpointer  str,
 		gint utf8_len;
 
 		/* Get next character of the word as UCS4 */
-		utf8_len = u8_strmbtouc (&unichar, &word[i]);
+		utf8_len = u8_strmbtouc (&unichar, (const guchar *) &word[i]);
 
 		/* Invalid UTF-8 character or end of original string. */
 		if (utf8_len <= 0) {
@@ -249,12 +253,12 @@ process_word_utf8 (TrackerParser         *parser,
 		/* Casefold and NFKD normalization in output.
 		 * NOTE: if the output buffer is not big enough, u8_casefold will
 		 * return a newly-allocated buffer. */
-		normalized = u8_casefold ((const uint8_t *)word,
-		                          length,
-		                          uc_locale_language (),
-		                          UNINORM_NFKD,
-		                          word_buffer,
-		                          &new_word_length);
+		normalized = (gchar*) u8_casefold ((const uint8_t *)word,
+		                                   length,
+		                                   uc_locale_language (),
+		                                   UNINORM_NFKD,
+		                                   (guchar *) word_buffer,
+		                                   &new_word_length);
 
 		/* Case folding + Normalization failed, ignore this word */
 		g_return_val_if_fail (normalized != NULL, NULL);
@@ -275,7 +279,7 @@ process_word_utf8 (TrackerParser         *parser,
 
 		normalized = length > WORD_BUFFER_LENGTH ? g_malloc (length + 1) : word_buffer;
 
-		for (i = 0; i < length; i++) {
+		for (i = 0; i < (gsize) length; i++) {
 			normalized[i] = g_ascii_tolower (word[i]);
 		}
 
@@ -345,7 +349,7 @@ parser_next (TrackerParser *parser,
 
 	/* Loop to look for next valid word */
 	while (!processed_word &&
-	       parser->cursor < parser->txt_size) {
+	       parser->cursor < (gsize) parser->txt_size) {
 		TrackerParserWordType type;
 		gsize truncated_length;
 		gboolean is_allowed;
@@ -424,15 +428,12 @@ parser_next (TrackerParser *parser,
 }
 
 TrackerParser *
-tracker_parser_new (TrackerLanguage *language)
+tracker_parser_new (void)
 {
 	TrackerParser *parser;
 
-	g_return_val_if_fail (TRACKER_IS_LANGUAGE (language), NULL);
-
 	parser = g_new0 (TrackerParser, 1);
-
-	parser->language = g_object_ref (language);
+	parser->language = tracker_language_new (NULL);
 
 	return parser;
 }
@@ -541,3 +542,106 @@ tracker_parser_next (TrackerParser *parser,
 	return str;
 }
 
+gpointer
+tracker_collation_init (void)
+{
+	/* Nothing to do */
+	return NULL;
+}
+
+void
+tracker_collation_shutdown (gpointer collator)
+{
+	/* Nothing to do */
+}
+
+gint
+tracker_collation_utf8 (gpointer      collator,
+                        gint          len1,
+                        gconstpointer str1,
+                        gint          len2,
+                        gconstpointer str2)
+{
+	gint result;
+	guchar *aux1;
+	guchar *aux2;
+
+	/* Note: str1 and str2 are NOT NUL-terminated */
+	aux1 = (len1 < MAX_STACK_STR_SIZE) ? g_alloca (len1+1) : g_malloc (len1+1);
+	aux2 = (len2 < MAX_STACK_STR_SIZE) ? g_alloca (len2+1) : g_malloc (len2+1);
+
+	memcpy (aux1, str1, len1); aux1[len1] = '\0';
+	memcpy (aux2, str2, len2); aux2[len2] = '\0';
+
+	result = u8_strcoll (aux1, aux2);
+
+	if (len1 >= MAX_STACK_STR_SIZE)
+		g_free (aux1);
+	if (len2 >= MAX_STACK_STR_SIZE)
+		g_free (aux2);
+	return result;
+}
+
+gunichar2 *
+tracker_parser_tolower (const gunichar2 *input,
+			gsize            len,
+			gsize           *len_out)
+{
+	return u16_tolower (input, len / 2, NULL, NULL, NULL, len_out);
+}
+
+gunichar2 *
+tracker_parser_toupper (const gunichar2 *input,
+                        gsize            len,
+                        gsize           *len_out)
+{
+	return u16_toupper (input, len / 2, NULL, NULL, NULL, len_out);
+}
+
+gunichar2 *
+tracker_parser_casefold (const gunichar2 *input,
+			 gsize            len,
+			 gsize           *len_out)
+{
+	return u16_casefold (input, len / 2, NULL, NULL, NULL, len_out);
+}
+
+gunichar2 *
+tracker_parser_normalize (const gunichar2 *input,
+			  GNormalizeMode   mode,
+			  gsize            len,
+			  gsize           *len_out)
+{
+	uninorm_t nf;
+
+	if (mode == G_NORMALIZE_NFC)
+		nf = UNINORM_NFC;
+	else if (mode == G_NORMALIZE_NFD)
+		nf = UNINORM_NFD;
+	else if (mode == G_NORMALIZE_NFKC)
+		nf = UNINORM_NFKC;
+	else if (mode == G_NORMALIZE_NFKD)
+		nf = UNINORM_NFKD;
+	else
+		g_assert_not_reached ();
+
+	return u16_normalize (nf, input, len / 2, NULL, len_out);
+}
+
+gunichar2 *
+tracker_parser_unaccent (const gunichar2 *input,
+			 gsize            len,
+			 gsize           *len_out)
+{
+	gunichar2 *zOutput;
+	gsize written = 0;
+
+	zOutput = u16_normalize (UNINORM_NFKD, input, len, NULL, &written);
+
+	/* Unaccenting is done in place */
+	tracker_parser_unaccent_nfkd_string (zOutput, &written);
+
+	*len_out = written;
+
+	return zOutput;
+}
